@@ -96,8 +96,7 @@ class Repository:
             if key in KEYWORDS_TO_NOT_FLATTEN or not isinstance(value, (list, tuple)):
                 data_to_send.append((key, value))
             else:
-                for item in value:
-                    data_to_send.append((key, item))
+                data_to_send.extend((key, item) for item in value)
         return data_to_send
 
     def set_certificate_authority(self, cacert: Optional[str]) -> None:
@@ -142,7 +141,10 @@ class Repository:
 
         with open(package.filename, "rb") as fp:
             data_to_send.append(
-                ("content", (package.basefilename, fp, "application/octet-stream"))
+                (
+                    "content",
+                    (package.basefilename, fp, "application/octet-stream"),
+                )
             )
             encoder = requests_toolbelt.MultipartEncoder(data_to_send)
 
